package com.onegini.oidc.security;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.oauth2.client.filter.OAuth2ClientContextFilter;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.LoginUrlAuthenticationEntryPoint;
import org.springframework.security.web.authentication.preauth.AbstractPreAuthenticatedProcessingFilter;

import com.onegini.oidc.IndexController;
import com.onegini.oidc.LogoutController;
import com.onegini.oidc.SampleSecuredController;

@Configuration
@EnableWebSecurity
public class SecurityConfiguration extends WebSecurityConfigurerAdapter {

  private static final String LOGIN_URL = "/login";

  @Bean
  public AuthenticationEntryPoint authenticationEntryPoint() {
    return new LoginUrlAuthenticationEntryPoint(LOGIN_URL);
  }

  @Bean
  public OpenIdConnectAuthenticationFilter openIdConnectAuthenticationFilter() {
    return new OpenIdConnectAuthenticationFilter(LOGIN_URL);
  }

/*  @Autowired
  public OAuth2ClientContextFilter oAuth2ClientContextFilter;*/

  @Override
  public void configure(final WebSecurity web) {
    web
        .ignoring()
        .antMatchers( "/favicon.ico");
  }

  @Override
  protected void configure(final HttpSecurity http) throws Exception {
    http.addFilterAfter(new OAuth2ClientContextFilter(), AbstractPreAuthenticatedProcessingFilter.class)
        .addFilterAfter(openIdConnectAuthenticationFilter(), OAuth2ClientContextFilter.class)
        .exceptionHandling().authenticationEntryPoint(authenticationEntryPoint())
        .and()
        .authorizeRequests()
        .antMatchers("/", "/logout", "/signout-callback-oidc").permitAll()
        .antMatchers( "/favicon.ico").permitAll()
        .antMatchers("/**").authenticated()
        .and()
        .logout()
        .logoutUrl(LogoutController.PAGE_LOGOUT)
        .logoutSuccessUrl(IndexController.PAGE_INDEX);

  }
}
